﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;


namespace PositionerControlLibrary
{
    public interface IPositionerCtrlInterface
    {
        string Version { get; }
        void Connect();
        void JogCCW(bool IsMove);
        void JogCW(bool IsMove);
        void Home();
        void AddPoint(float[] JointDataArr);
        void Start();
        void Stop();
        IPositionerModelService PositionerModelService { get; }
        void LoadPointFromTextFile();
        void UpdateTextFile();

    }

    public class PositionerController : IPositionerCtrlInterface
    {
        private readonly IPositionerModelService mPositionerService = null;
        public string Version { get; } = "5.0";
        public IPositionerModelService PositionerModelService { get { return mPositionerService; } }

        public void Connect()
        {
            mPositionerService.InitConnection();
        }

        public void JogCW(bool IsMove)
        {
            mPositionerService.ControlExtAxis(ModbusData.Cmd.JOG_CW, IsMove);
        }

        public void JogCCW(bool IsMove)
        {
            mPositionerService.ControlExtAxis(ModbusData.Cmd.JOG_CCW, IsMove);
        }
        public void Home()
        {
            mPositionerService.ControlExtAxis(ModbusData.Cmd.POS_ZERO, true);
        }
        public void UpdateTextFile()
        {
            PointDataFactory.SaveDataToFile(mPositionerService.PositionerCtrlService);
        }
        public void LoadPointFromTextFile()
        {
            PointDataFactory.GetFileData(mPositionerService.PositionerCtrlService);
        }

        public void AddPoint(float[] JointDataArr)
        {
            int cnt = mPositionerService.PositionerCtrlService.PositionerDataList.Count;
            int pos_idx = (cnt == 0) ? 0 : mPositionerService.PositionerCtrlService.PositionerDataList[cnt - 1].Index + 1;
            mPositionerService.PositionerCtrlService.AddPositionerDataList(pos_idx, JointDataArr);
            UpdateTextFile();
        }

        public void Start()
        {
            mPositionerService.ControlExtAxis(ModbusData.Cmd.POS_START, true);
        }
        public void Stop()
        {
            mPositionerService.ControlExtAxis(ModbusData.Cmd.STOP, true);
        }

        public PositionerController()
        {
            mPositionerService = new PositionerModel();
        }

    }
}
