﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using System.Xml.Serialization;

namespace PositionerControlLibrary
{
    public static class PointDataFactory
    {
        public static string FileName_Xml = "PointInformation.xml";
        public static string FileName_Txt = "..\\..\\..\\TextFiles\\PointInformation.txt";
        public static string Target_Folder = "..\\..\\..\\TextFiles";

        private static void CheckTextFolder()
        {
            if (Directory.Exists(Target_Folder)) { return; }

            Directory.CreateDirectory(Target_Folder);
        }

        public static void GetFileData(IPositionerCtrlService CtrlService)
        {
            CheckTextFolder();

            if (File.Exists(FileName_Txt) == false) 
            {
                using (var stream = File.Create(FileName_Txt))
                {
                    // Use stream
                }
            }

            File.Copy(FileName_Txt, FileName_Xml, true);

            if (File.Exists(FileName_Xml) == false) { return; }

            XmlSerializer serializer = new XmlSerializer(typeof(PointInformation));
            XmlReader reader = XmlReader.Create(FileName_Xml);

            PointInformation data = new PointInformation();

            try
            {

                if (serializer.CanDeserialize(reader) == false)
                {
                    reader.Close();
                    return;
                }

                data = (PointInformation)serializer.Deserialize(reader);
            }
            catch { }

            reader.Close();

            foreach (PointData p in data.PointDataList)
            {
                float[] joint_arr = new float[6] { p.J_1, p.J_2, p.J_3, p.J_4, p.J_5, p.J_6 };
                CtrlService.AddPositionerDataList(p.Index, p.J_7, joint_arr);
            }
        }

        public static void SaveDataToFile(IPositionerCtrlService Data)
        {
            // Creates an instance of the XmlSerializer class;
            // specifies the type of object to serialize.
            XmlSerializer serializer = new XmlSerializer(typeof(PointInformation));
            TextWriter writer = new StreamWriter(FileName_Xml);
            serializer.Serialize(writer, GenerateSerializeData(Data));
            writer.Close();

            if (File.Exists(FileName_Xml) == false) { File.Create(FileName_Xml); }
            File.Copy(FileName_Xml, FileName_Txt, true);
            File.Delete(FileName_Xml);
        }

        public static PointInformation GenerateSerializeData(IPositionerCtrlService Data)
        {
            PointInformation info = new PointInformation();

            foreach (IPositionerData point in Data.PositionerDataList)
            {
                PointData pointData = new PointData() 
                { 
                    Index = point.Index,
                    J_1 = point.JointData[0], 
                    J_2 = point.JointData[1], 
                    J_3 = point.JointData[2], 
                    J_4 = point.JointData[3], 
                    J_5 = point.JointData[4], 
                    J_6 = point.JointData[5], 
                    J_7 = point.Angle 
                };
                info.PointDataList.Add(pointData);
            }

            return info;
        }
    }

}
