﻿using HslCommunication.Core;
using PositionerControlLibrary;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Reflection.Metadata;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows;


namespace PositionerControlLibrary
{
    public interface IPositionerCtrlService
    {
        string ConnectState { get; set; }
        ushort TotalExtAxis { get; }
        string TargetAngleForDisplay { get; set; }
        short CurrentAngle { get; set; }
        void SetAngleToTarget(int ListIndex);
        void InitPositionerDataList();
        void AddPositionerDataList(int PosIdx, float[] JointDataArr);
        void AddPositionerDataList(int PosIdx, int Position, float[] JointDataArr);
        void AddModbusDataList(VariableData Data);
        void ClearModbusDataList();
        BindingList<IPositionerData> PositionerDataList { get; }
        bool Estop { get; set; }
        bool Alarm { get; set; }
        bool InPos { get; set; }
        bool PositioningOk { get; set; }
        bool OriOk { get; set; }
        bool InZero { get; set; }
        short Angle_Total { get; set; }
        short Angle_Cycle { get; set; }
        string AngleForDisplay { get; set; }
        double Target_Speed { get; set; }
        int TargetAngle { get; }
        bool SetTargetAngleToInt(string SetValue, out int RtnSetValue);

        double Target_VelocityForSlider { get; set; }
        double Target_Velocity { get; set; }
        double Radius { get; set; }
        void UpdateVelocity(double Velocity, double R);
        void UpdateSliderValue();
        List<VariableMockData> VariableMockDataList { get;}
    }

    public class VariableMockData : INotifyPropertyChanged
    {
        string mCurValue = "";
        bool mSelect = false;
        public bool Select 
        {
            get { return mSelect; }
            set { mSelect = value; NotifyPropertyChanged("Select"); }
        }
        public string Name { get; set; }
        public uint Address { get; set; }
        public VariableTypeDef DataType { get; set; }
        public string CurValue 
        {
            get { return mCurValue; }
            set { mCurValue = value; NotifyPropertyChanged("CurValue"); } 
        }

        public event PropertyChangedEventHandler PropertyChanged;

        private void NotifyPropertyChanged(string propertyName = "")
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
    }

    class PositionerCtrlData: IPositionerCtrlService, INotifyPropertyChanged
    {
        private const short MAX_SPEED = 5000;
        private int mTargetAngle = 0;
        private bool mEstop = false;
        private bool mAlarm = false;
        private bool mInPos = false;
        private bool mPositioningOk = false;
        private bool mOriOk = false;
        private bool mInZero = false;
        private short mAngle_Total = 0;
        private short mAngle_Cycle = 0;
        private double mTarget_Speed = 20.00;
        private short mCurrentAngle = 0;
        private string mConnectState = "None";
        private double mTarget_Velocity = 0.0;
        private double mRadius = 0.0;
        private double mTarget_VelocityForSlider = 0.0;

        List<VariableMockData> mVariableMockDataList = new List<VariableMockData>();

        public List<VariableMockData> VariableMockDataList { get { return mVariableMockDataList; } }

        public void ClearModbusDataList()
        {
            mVariableMockDataList.Clear(); 
        }

        public void AddModbusDataList(VariableData Data)
        {
            mVariableMockDataList.Add(new VariableMockData() { Name = Data.Name, Address = Data.Address, DataType = Data.DataType, CurValue = "" });
        }

        public double Target_VelocityForSlider 
        {
            get { return mTarget_VelocityForSlider; } 
            set
            {
                mTarget_VelocityForSlider = value;
                mTarget_Speed = Math.Round(mTarget_VelocityForSlider, 2);
                NotifyPropertyChanged("Target_Speed");
                NotifyPropertyChanged("Target_VelocityForSlider");
            }
        }


        public void UpdateVelocity(double Velocity, double R)
        {
            if (R == 0) { return; }
            Target_Speed = 60 * Velocity / 3.14159 / R;
        }

        public void UpdateSliderValue()
        {
            Target_VelocityForSlider = Target_Speed;
        }

        public double Target_Velocity 
        { 
            get
            {
                return mTarget_Velocity;
            }
            set
            {
                mTarget_Velocity = value;
                UpdateVelocity(mTarget_Velocity, mRadius);
                UpdateSliderValue();
                NotifyPropertyChanged("Target_VelocityForSlider");
            }
        }
        public double Radius
        {
            get
            {
                return mRadius;
            }
            set
            {
                mRadius = value;
                UpdateVelocity(mTarget_Velocity, mRadius);
                NotifyPropertyChanged("Target_Speed");
            }
        }

        public ushort TotalExtAxis { get; private set; } = 3;

        public int TargetAngle { get { return mTargetAngle; } }
        public string ConnectState
        {
            get { return mConnectState; }
            set { mConnectState = value; NotifyPropertyChanged("ConnectState"); }
        }

        public double Target_Speed 
        { 
            get 
            { 
                return mTarget_Speed; 
            } 
            set 
            { 
                mTarget_Speed = value;
                UpdateSliderValue();
                NotifyPropertyChanged("Target_Speed");
            }
        }

        public short Angle_Total
        {
            get { return mAngle_Total; }
            set { mAngle_Total = value; NotifyPropertyChanged("Angle_Total"); NotifyPropertyChanged("AngleForDisplay"); }
        }
        public short Angle_Cycle
        {
            get { return mAngle_Cycle; }
            set { mAngle_Cycle = value; NotifyPropertyChanged("Angle_Cycle"); NotifyPropertyChanged("AngleForDisplay"); }
        }

        public string AngleForDisplay 
        {
            get 
            { 
                return string.Format("{0} ({1})", ((double)Angle_Total * 0.1).ToString("#0.0"), ((double)Angle_Cycle * 0.1).ToString("#0.0")); 
            }
            set {  }
        }


        private string mTargetAngleForDisplay = "0.0";
        public string TargetAngleForDisplay
        {
            get 
            { 
                return mTargetAngleForDisplay; 
            }
            set 
            {
                int rtn_value = 0;
                //if (NumericOnly(value) == false) { return; }
                if (false == SetTargetAngleToInt(value, out rtn_value)) { return; }
                mTargetAngle = rtn_value;

                mTargetAngleForDisplay = value;
                NotifyPropertyChanged("TargetAngleForDisplay"); 
            }
        }

        private bool NumericOnly(string ValueStr)
        {
            Regex reg = new Regex("[^0-9.]+-");
            return !reg.IsMatch(ValueStr);
        }

        public bool SetTargetAngleToInt(string SetValue, out int RtnSetValue)
        {
            double rtn_double = 0.0;
            RtnSetValue = 0;
            int dot_idx = SetValue.IndexOf('.');
            if (dot_idx < (SetValue.Length - 2) && dot_idx > 0) { return false; }
            if (false == double.TryParse(SetValue, out rtn_double)) { return false; }
            RtnSetValue = (int)(rtn_double * 10);
            return true;
        }


        public short CurrentAngle
        {
            get { return mCurrentAngle; }
            set { mCurrentAngle = value; NotifyPropertyChanged("CurrentAngle"); }
        }
        public bool Estop
        {
            get { return mEstop; }
            set { mEstop = value; NotifyPropertyChanged("Estop"); }
        }
        public bool Alarm
        {
            get { return mAlarm; }
            set { mAlarm = value; NotifyPropertyChanged("Alarm"); }
        }
        public bool InPos
        {
            get { return mInPos; }
            set { mInPos = value; NotifyPropertyChanged("InPos"); }
        }
        public bool PositioningOk
        {
            get { return mPositioningOk; }
            set { mPositioningOk = value; NotifyPropertyChanged("PositioningOk"); }
        }
        public bool OriOk
        {
            get { return mOriOk; }
            set { mOriOk = value; NotifyPropertyChanged("OriOk"); }
        }
        public bool InZero
        {
            get { return mInZero; }
            set { mInZero = value; NotifyPropertyChanged("InZero"); }
        }

        public BindingList<IPositionerData> PositionerDataList { get; private set; } = new BindingList<IPositionerData>();

        public event PropertyChangedEventHandler PropertyChanged;

        private void NotifyPropertyChanged(string propertyName = "")
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        public void SetAngleToTarget(int ListIndex)
        {
            mTargetAngle = PositionerDataList[ListIndex].Angle;
            mTargetAngleForDisplay = ((double)(mTargetAngle * 0.1)).ToString();
            NotifyPropertyChanged("TargetAngleForDisplay");
        }

        public void InitPositionerDataList()
        {
            PositionerDataList.Clear();
        }

        public void AddPositionerDataList(int PosIdx, float[] JointDataArr)
        {
            IPositionerData data = new PositionerData()
            {
                Name = string.Format("A{0}", PosIdx.ToString()),
                Angle = CurrentAngle,
                Index = PosIdx,
                JointData = JointDataArr
            };
            PositionerDataList.Add(data);
        }

        public void AddPositionerDataList(int PosIdx, int Position, float[] JointDataArr)
        {
            IPositionerData data = new PositionerData()
            {
                Name = string.Format("A{0}", PosIdx.ToString()),
                Angle = Position,
                Index = PosIdx,
                JointData = JointDataArr
            };
            PositionerDataList.Add(data);
        }
    }

}
