﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PositionerControlLibrary
{
    public interface IPositionerData
    {
        int Index { get; set; }
        string Name { get; set; }
        int Angle { get; set; }
        string AngleString { get; set; }
        bool Angle_Edit { get; set; }
        bool Angle_Edit_Valid { get; set; }
        float[] JointData { get; set; }
    }


    class PositionerData : IPositionerData, INotifyPropertyChanged
    {
        private bool mAngle_Edit = false;
        private bool mAngle_Edit_Valid = true;
        private int mAngle = 0;
        public int Index { get; set; } = 0;
        public string Name { get; set; } = "";
        public int Angle
        {
            get { return mAngle; }
            set 
            { 
                mAngle = value;
                mAngleString = ((double)(value * 0.1)).ToString();

                NotifyPropertyChanged("Angle"); 
                NotifyPropertyChanged("AngleString"); 
            }
        }


        string mAngleString = "";

        public string AngleString
        {
            get 
            { 
                return mAngleString; 
            }
            set 
            {
                double rtn_double = 0.0;
                if (false == double.TryParse(value, out rtn_double)) { return; } 

                mAngle = (int)(rtn_double * 10);
                mAngleString = value;
                NotifyPropertyChanged("AngleString"); 
                NotifyPropertyChanged("Angle"); 
            }
        }

        public bool Angle_Edit
        {
            get { return mAngle_Edit; }
            set { mAngle_Edit = value; NotifyPropertyChanged("Angle_Edit"); NotifyPropertyChanged("Angle_Edit_Valid"); }
        }

        public bool Angle_Edit_Valid
        {
            get { return mAngle_Edit_Valid & mAngle_Edit; }
            set { mAngle_Edit_Valid = value; NotifyPropertyChanged("Angle_Edit_Valid"); }
        }

        public float[] JointData { get; set; } = new float[6];


        public event PropertyChangedEventHandler PropertyChanged;

        private void NotifyPropertyChanged(string propertyName = "")
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }


    }

}
