﻿using HslCommunication.ModBus;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PositionerControlLibrary
{
    public interface IPositionerModelService
    {
        bool IsConnected { get; }
        void InitConnection();
        void UpdateStatus();
        void ControlExtAxis(ModbusData.Cmd Cmd, bool Value);
        IPositionerCtrlService PositionerCtrlService { get; }

        void WriteModbusData();
        void ReadModbusData();
    }

    class PositionerModel : IPositionerModelService
    {
        public IPositionerCtrlService PositionerCtrlService { get { return mIPositionerCtrlService; } }
        public bool IsConnected { get { return mIsConnectServer; } }

        private IPositionerCtrlService mIPositionerCtrlService = new PositionerCtrlData();

        private ModbusTcpNet mbusTcpClient = new ModbusTcpNet("192.168.1.5");
        private bool mIsConnectServer = false;


        public void InitConnection()
        {
            if (mIPositionerCtrlService.ConnectState == "Normal") { return; }

            mbusTcpClient.ConnectClose();
            mIsConnectServer = mbusTcpClient.ConnectServer().IsSuccess;

            if (mIsConnectServer == true)
            {
                mIPositionerCtrlService.ConnectState = "Normal";
                return;
            }
            mIPositionerCtrlService.ConnectState = "Error";
        }


        public void UpdateStatus()
        {
            if (mIsConnectServer == false || mIPositionerCtrlService.ConnectState != "Normal") { return; }
            try
            {
                mIPositionerCtrlService.Estop = mbusTcpClient.ReadBool(ModbusData.GetModbusAddress(ModbusData.Cmd.ESTOP), 1).Content[0];
                mIPositionerCtrlService.Alarm = mbusTcpClient.ReadBool(ModbusData.GetModbusAddress(ModbusData.Cmd.ALM), 1).Content[0];
                mIPositionerCtrlService.InPos = mbusTcpClient.ReadBool(ModbusData.GetModbusAddress(ModbusData.Cmd.IN_POS), 1).Content[0];
                mIPositionerCtrlService.PositioningOk = mbusTcpClient.ReadBool(ModbusData.GetModbusAddress(ModbusData.Cmd.POS_OK), 1).Content[0];
                mIPositionerCtrlService.OriOk = mbusTcpClient.ReadBool(ModbusData.GetModbusAddress(ModbusData.Cmd.ORG_OK), 1).Content[0];
                mIPositionerCtrlService.InZero = mbusTcpClient.ReadBool(ModbusData.GetModbusAddress(ModbusData.Cmd.IN_ZERO), 1).Content[0];

                mIPositionerCtrlService.Angle_Total = mbusTcpClient.ReadInt16(ModbusData.GetModbusAddress(ModbusData.Cmd.CUR_ANGLE_TOTAL), 1).Content[0];
                mIPositionerCtrlService.Angle_Cycle = mbusTcpClient.ReadInt16(ModbusData.GetModbusAddress(ModbusData.Cmd.CUR_ANGLE_CYCLE), 1).Content[0];

            }
            catch (Exception ex) { mIPositionerCtrlService.ConnectState = "Error"; }
        }

        public void WriteModbusData()
        {
            foreach(VariableMockData mock in mIPositionerCtrlService.VariableMockDataList)
            {
                if (mock.Select == false) { continue; }
                switch(mock.DataType)
                {
                    case VariableTypeDef.BOOL:
                        bool rtn_bool = false;
                        if (false == bool.TryParse(mock.CurValue, out rtn_bool)) { continue; }
                        mbusTcpClient.Write(mock.Address.ToString(), rtn_bool);
                        break;

                    case VariableTypeDef.WORD:
                        ushort rtn_ushort = 0;
                        if (false == ushort.TryParse(mock.CurValue, out rtn_ushort)) { continue; }
                        mbusTcpClient.Write(mock.Address.ToString(), rtn_ushort);
                        break;
                }
            }
        }
        public void ReadModbusData()
        {
            foreach(VariableMockData mock in mIPositionerCtrlService.VariableMockDataList)
            {
                switch (mock.DataType)
                {
                    case VariableTypeDef.BOOL:
                        mock.CurValue = mbusTcpClient.ReadBool(mock.Address.ToString(), 1).Content[0].ToString();
                        break;

                    case VariableTypeDef.WORD:
                        mock.CurValue = mbusTcpClient.ReadInt16(mock.Address.ToString(), 1).Content[0].ToString();
                        break;
                }
            }
        }

        public void ControlExtAxis(ModbusData.Cmd Cmd, bool Value)
        {
            if (mIsConnectServer == false || mIPositionerCtrlService.ConnectState != "Normal") { return; }

            try
            {
                if (mIPositionerCtrlService.Target_Speed > 50)
                {
                    mIPositionerCtrlService.Target_Speed = 50;
                }

                if (Cmd != ModbusData.Cmd.STOP)
                {
                    mbusTcpClient.Write(ModbusData.GetModbusAddress(ModbusData.Cmd.STOP), false);
                }
                if (Cmd == ModbusData.Cmd.POS_START)
                {
                    mbusTcpClient.Write(ModbusData.GetModbusAddress(ModbusData.Cmd.POS_ZERO), false);
                    mbusTcpClient.Write(ModbusData.GetModbusAddress(ModbusData.Cmd.RTN_ORG), false);
                    mbusTcpClient.Write(ModbusData.GetModbusAddress(ModbusData.Cmd.SET_ZERO), false);
                    mbusTcpClient.Write(ModbusData.GetModbusAddress(Cmd), false);

                    short high_value = (short)((mIPositionerCtrlService.TargetAngle >> 16) & 0xFFFF);
                    short low_value = (short)(mIPositionerCtrlService.TargetAngle & 0xFFFF);


                    mbusTcpClient.Write(ModbusData.GetModbusAddress(ModbusData.Cmd.TAR_ANGLE_LOW), low_value);
                    mbusTcpClient.Write(ModbusData.GetModbusAddress(ModbusData.Cmd.TAR_ANGLE_HIGH), high_value);
                    mbusTcpClient.Write(ModbusData.GetModbusAddress(ModbusData.Cmd.TAR_SPEED), (ushort)(Math.Round(mIPositionerCtrlService.Target_Speed, 2) * 100));
                }
                if ((Cmd == ModbusData.Cmd.JOG_CW)
                    || (Cmd == ModbusData.Cmd.JOG_CCW))
                {
                    mbusTcpClient.Write(ModbusData.GetModbusAddress(ModbusData.Cmd.JOG_SPEED), (ushort)(Math.Round(mIPositionerCtrlService.Target_Speed, 2) * 100));
                }
                mbusTcpClient.Write(ModbusData.GetModbusAddress(Cmd), Value);
            }
            catch (Exception ex) { mIPositionerCtrlService.ConnectState = "Error"; }
        }
    }
}
