using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using PositionerControlLibrary;
using TMcraft;


namespace PositionerControlNode
{
    /// <summary>
    /// Interaction logic for UserControl1.xaml
    /// </summary>
    public partial class NodeUserControl : UserControl, ITMcraftNodeEntry
    {
        private TMcraftNodeAPI mTMcraftNodeAPI = null;
        private ScriptWriteProvider mScriptWriteProvider = null;
        private IPositionerCtrlService mPositionerCtrlService = null;
        private IPositionerCtrlInterface mControlService = new PositionerController();

        private string mTargetPosition = "";
        private string mTargetPositionForSaveData = "";

        public NodeUserControl()
        {
            InitializeComponent();

            mPositionerCtrlService = mControlService.PositionerModelService.PositionerCtrlService;
            DataContext = mPositionerCtrlService;
            grid_angle.ItemsSource = mPositionerCtrlService.PositionerDataList;
            mControlService.LoadPointFromTextFile();

            int count = mPositionerCtrlService.PositionerDataList.Count;
            tb_target_pos.Text = (mPositionerCtrlService.PositionerDataList[count - 1].Angle * 0.1).ToString();
        }

        public void InitializeNode(TMcraftNodeAPI Api)
        {
            mTMcraftNodeAPI = Api;
            string rtn_data = "";
            mTMcraftNodeAPI.DataStorageProvider.GetData("TargetPos", out rtn_data);

            if (rtn_data == "") { return; }
            tb_target_pos.Text = rtn_data;
        }

        public void InscribeScript(ScriptWriteProvider Provider)
        {
            if (mTargetPosition == "") { return; }

            mScriptWriteProvider = Provider;
            List<string> script_list = new List<string>();

            int rtn_target_pos = 0;
            if (false == Int32.TryParse(mTargetPosition, out rtn_target_pos))
            {
                return;
            }

            short high_value = (short)((rtn_target_pos >> 16) & 0xFFFF);
            short low_value = (short)(rtn_target_pos & 0xFFFF);

            script_list.Add("ModbusTCP mbus1 = \"192.168.1.5\"");
            script_list.Add("modbus_open(\"mbus1\")");
            script_list.Add("mbus1.Preset(\"TarPos1\", 0, \"RO\", 4596, \"int16\")");
            script_list.Add("mbus1.Preset(\"TarPos2\", 0, \"RO\", 4597, \"int16\")");
            script_list.Add("mbus1.Preset(\"StartMov\", 0, \"DO\", 2051, \"bool\")");
            script_list.Add("mbus1.Preset(\"PositioningOK\", 0, \"DI\", 2061, \"bool\")");
            script_list.Add(string.Format("modbus_write(\"mbus1\", \"TarPos1\", {0})", low_value.ToString()));
            script_list.Add(string.Format("modbus_write(\"mbus1\", \"TarPos2\", {0})", high_value.ToString()));
            script_list.Add("modbus_write(\"mbus1\", \"StartMov\", True)");
            script_list.Add("while (True)");
            script_list.Add("{");
            script_list.Add("Sleep(500)");
            script_list.Add("if (modbus_read(\"mbus1\", \"PositioningOK\") == True)");
            script_list.Add("{");
            script_list.Add("break");
            script_list.Add("}");
            script_list.Add("}");

            foreach(string script in script_list) { mScriptWriteProvider?.AppendLine(script); }

            Dictionary<string, string> save_data = new Dictionary<string, string>();
            save_data.Add("TargetPos", mTargetPositionForSaveData);
            mTMcraftNodeAPI?.DataStorageProvider.SaveData(save_data);
            mTMcraftNodeAPI?.Close();
        }

        private void btn_OK_Click(object sender, RoutedEventArgs e)
        {
            double rtn_double = 0.0;
            if (false == double.TryParse(tb_target_pos.Text, out rtn_double))
            {
                MessageBox.Show(string.Format("Target position invalid ({0})", tb_target_pos.Text));
           
                mTMcraftNodeAPI?.Close();
            }
            mTargetPosition = (rtn_double * 10).ToString();
            mTargetPositionForSaveData = (rtn_double).ToString();
            mTMcraftNodeAPI?.Close();

        }

        private void btn_Cancel_Click(object sender, RoutedEventArgs e)
        {
            mTMcraftNodeAPI?.Close();
        }

        private void btn_copy_Click(object sender, RoutedEventArgs e)
        {
            if (grid_angle.SelectedIndex == -1) { return; }
            if (grid_angle.SelectedIndex >= mPositionerCtrlService.PositionerDataList.Count) { return; }

            tb_target_pos.Text = (mPositionerCtrlService.PositionerDataList[grid_angle.SelectedIndex].Angle * 0.1).ToString();
        }
    }
}