using PositionerControlLibrary;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;
using TMcraft;
using System.Windows.Interop;
using System.Runtime.InteropServices;
using System.Diagnostics;
using System.Text.RegularExpressions;
using System.Reflection.Metadata;

namespace PositionerControlToolbar
{
    /// <summary>
    /// Interaction logic for UserControl1.xaml
    /// </summary>
    public partial class ToolbarUseControl : UserControl, ITMcraftToolbarEntry 
    {
        private TMcraftToolbarAPI mTMcraftToolbarAPI = null;
        private IPositionerCtrlService mPositionerCtrlService = null;
        private IPositionerCtrlInterface mControlService = new PositionerController();
        private static DispatcherTimer readDataTimer = new DispatcherTimer();
        private GeneralSettingPage mGeneralSettingPage = null;

        private int mOldAngle = 0;

        public ToolbarUseControl()
        {
            InitializeComponent();
            mPositionerCtrlService = mControlService.PositionerModelService.PositionerCtrlService;
            DataContext = mPositionerCtrlService;
            grid_angle.ItemsSource = mPositionerCtrlService.PositionerDataList;

            mControlService.LoadPointFromTextFile();

            mGeneralSettingPage = new GeneralSettingPage(mControlService.PositionerModelService);

            if (mPositionerCtrlService == null)
            {
                AddPositionWithJointAngle();
                return;
            }

            if (mPositionerCtrlService.PositionerDataList == null)
            {
                AddPositionWithJointAngle();
            }

            if (mPositionerCtrlService.PositionerDataList?.Count == 0)
            {
                AddPositionWithJointAngle();
            }

            mPositionerCtrlService.UpdateSliderValue();
            UpdateRadioBtnState();
        }

        public void InitializeToolbar(TMcraftToolbarAPI Api)
        {
            mTMcraftToolbarAPI = Api;
            InitConnect();
        }

        private void btn_connect_Click(object sender, RoutedEventArgs e)
        {
            InitConnect();
        }

        private void InitConnect()
        {
            mControlService.Connect();

            if (mPositionerCtrlService.ConnectState == "Normal")
            {
                readDataTimer.Tick += new EventHandler(TimerFunction);
                readDataTimer.Interval = new TimeSpan(0, 0, 0, 0, 500);
                readDataTimer.Start();
            }
        }

        public void TimerFunction(object? sender, EventArgs e)
        {
            mPositionerCtrlService.CurrentAngle = mPositionerCtrlService.Angle_Total;
            UpdateStatus();
        }

        private void UpdateStatus()
        {
            mControlService.PositionerModelService?.UpdateStatus();
        }

        private void btn_home_Click(object sender, RoutedEventArgs e)
        {
            mControlService.Home();
        }

        private void btn_add_point_Click(object sender, RoutedEventArgs e)
        {
            AddPositionWithJointAngle();
        }


        private void AddPositionWithJointAngle()
        {
            float[] rtn_joint = new float[6] { 0, 0, 0, 0, 0, 0 };

            if (mTMcraftToolbarAPI != null)
            {
                mTMcraftToolbarAPI.RobotStatusProvider.GetCurrentPoseByJointAngle(out rtn_joint);
            }

            mControlService.AddPoint(rtn_joint);
        }

        private void btn_start_Click(object sender, RoutedEventArgs e)
        {
            mControlService.Start();
        }

        private void btn_stop_Click(object sender, RoutedEventArgs e)
        {
            mControlService.Stop();
        }

        private void btn_jog_ccw_PreviewMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            mControlService.JogCCW(true);
        }

        private void btn_jog_ccw_PreviewMouseUp(object sender, MouseButtonEventArgs e)
        {
            mControlService.JogCCW(false);
        }

        private void btn_jog_cw_PreviewMouseUp(object sender, MouseButtonEventArgs e)
        {
            mControlService.JogCW(false);
        }

        private void btn_jog_cw_PreviewMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            mControlService.JogCW(true);
        }

        private void btn_copy_Click(object sender, RoutedEventArgs e)
        {
            if (grid_angle.SelectedIndex == -1) { return; }
            if (grid_angle.SelectedIndex >= mPositionerCtrlService.PositionerDataList.Count) { return; }
            mPositionerCtrlService.SetAngleToTarget(grid_angle.SelectedIndex);
        }

        private void btn_edit_Click(object sender, RoutedEventArgs e)
        {
            int row_idx = grid_angle.SelectedIndex;
            if (row_idx >= mPositionerCtrlService.PositionerDataList.Count) { return; }
            mOldAngle = mPositionerCtrlService.PositionerDataList[grid_angle.SelectedIndex].Angle;
            mPositionerCtrlService.PositionerDataList[grid_angle.SelectedIndex].Angle_Edit = true;

        }

        private void btn_delete_Click(object sender, RoutedEventArgs e)
        {
            if (grid_angle.SelectedIndex == -1) { return; }
            if (grid_angle.SelectedIndex >= mPositionerCtrlService.PositionerDataList.Count) { return; }

            mPositionerCtrlService.PositionerDataList.RemoveAt(grid_angle.SelectedIndex);
            mControlService.UpdateTextFile();
        }

        private void btn_ok_Click(object sender, RoutedEventArgs e)
        {
            if (grid_angle.SelectedIndex == -1) { return; }
            if (grid_angle.SelectedIndex >= mPositionerCtrlService.PositionerDataList.Count) { return; }
            mPositionerCtrlService.PositionerDataList[grid_angle.SelectedIndex].Angle_Edit = false;
            mControlService.UpdateTextFile();
        }

        private void btn_cancel_Click(object sender, RoutedEventArgs e)
        {
            if (grid_angle.SelectedIndex == -1) { return; }
            if (grid_angle.SelectedIndex >= mPositionerCtrlService.PositionerDataList.Count) { return; }
            mPositionerCtrlService.PositionerDataList[grid_angle.SelectedIndex].Angle_Edit = false;
            mPositionerCtrlService.PositionerDataList[grid_angle.SelectedIndex].Angle = mOldAngle;

        }

        private void tb_angle_TextChanged(object sender, TextChangedEventArgs e)
        {
            TextBox tb = (sender as TextBox);
            if (tb == null) { return; }

            if (grid_angle.SelectedIndex == -1) { return; }
            if (grid_angle.SelectedIndex >= mPositionerCtrlService.PositionerDataList.Count) { return; }

            int rtn_value = 0;

            if (false == mPositionerCtrlService.SetTargetAngleToInt(tb.Text, out rtn_value))
            {
                mPositionerCtrlService.PositionerDataList[grid_angle.SelectedIndex].Angle_Edit_Valid = false; return;
            }

            mPositionerCtrlService.PositionerDataList[grid_angle.SelectedIndex].Angle_Edit_Valid = true;
        }

        private void rad_rpm_Click(object sender, RoutedEventArgs e)
        {
            UpdateRadioBtnState();
        }

        private void UpdateRadioBtnState()
        {
            bool target_status = false;
            if (rad_vel.IsChecked != null)
            {
                target_status = (bool)rad_vel.IsChecked;
            }
            tb_vel.IsEnabled = target_status;
            tb_r.IsEnabled = target_status;
            tb_speed.IsEnabled = !(target_status);
            UpdateTargetVelocity();
        }

        private void UpdateTargetVelocity()
        {
            if (rad_vel.IsChecked == false) { return; }

            double rtn_vel_double = 0.0;
            double rtn_r_double = 0.0;

            if (false == double.TryParse(tb_vel.Text, out rtn_vel_double))
            {
                return;
            }

            if (false == double.TryParse(tb_r.Text, out rtn_r_double))
            {
                return;
            }

            mPositionerCtrlService.UpdateVelocity(rtn_vel_double, rtn_r_double);
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            Window general_win = new Window();
            general_win.Content = mGeneralSettingPage;
            general_win.Width = 500; 
            general_win.Height = 500;
            general_win.WindowStyle = WindowStyle.ToolWindow;
            general_win.ShowInTaskbar = false;
            Window parent_win = GetParentWindow(this.Parent);
            if (parent_win != null) 
            {
                general_win.WindowStartupLocation = WindowStartupLocation.Manual;

                general_win.Top = parent_win.Top;
                general_win.Left = parent_win.Left - general_win.Width;
            }
            else
            {
                general_win.WindowStartupLocation = WindowStartupLocation.CenterScreen;
            }

            general_win.Show();
        }

        private Window GetParentWindow(DependencyObject ControlParent)
        {
            if (ControlParent == null) { return null; }

            switch (ControlParent)
            {
                case Window: return (ControlParent as Window);
                case Grid: return GetParentWindow((ControlParent as Grid).Parent);
                case Panel: return GetParentWindow((ControlParent as Panel).Parent);
                case ScrollViewer: return GetParentWindow((ControlParent as ScrollViewer).Parent);
                case ContentControl: return GetParentWindow((ControlParent as ContentControl).Parent);
                default: return null;
            }
        }
    }

    public class TextToVisibilityConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            if (value.ToString() == "False")
            {
                return Visibility.Visible;
            }
            return Visibility.Collapsed;
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    public class ReverTextToVisibilityConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            if (value.ToString() == "True")
            {
                return Visibility.Visible;
            }
            return Visibility.Collapsed;
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }
}